package migl.lisp;

import migl.lisp.operator.DefineOperator;

/**
 * Simple factory to access the interpreter implementation.
 * 
 * @author leberre
 *
 */
public class LispFactory {
	
	private static Lisp interpreter;

    private LispFactory() {
        // do nothing
    }

    /**
     * Create a new instance of the interpreter.
     * Reset LispElement cache and define variable
     * 
     * @return a new lisp interpreter.
     */
    public static Lisp makeIntepreter() {
    	LispElement.clear();
    	DefineOperator.clear();
    	if(interpreter == null) {
    		interpreter = new LispImpl();
    	}
        return interpreter;
    }
    
    /**
     * Create a new instance of the interpreter.
     * 
     * @return a new lisp interpreter.
     */
    public static Lisp getInterpreter() {
    	if(interpreter == null) {
    		interpreter = new LispImpl();
    	}
        return interpreter;
    }
}
